;; GleamThin/theme.jl
;; Copyright 2000, 2003 Rowan Kerr <rowan@stasis.org>

(require 'gradient)

(defgroup gradient "GleamThin Customization"
  :group appearance
)

(defcustom gradient:gradient-type 'horizontal
  "Direction of gradient in `gradient' frame style."
  :type symbol
  :options (horizontal vertical diagonal)
  :group (appearance gradient)
  :after-set after-setting-frame-option
)

(defcustom gradient:normal-from-color (get-color "#dedede")
  "`From' color of inactive frames in `gradient' frame style."
  :type color
  :group (appearance gradient)
  :after-set after-setting-frame-option
)

(defcustom gradient:normal-to-color (get-color "#9e9e9e")
  "`To' color of inactive frames in `gradient' frame style."
  :type color
  :group (appearance gradient)
  :after-set after-setting-frame-option
)

(defcustom gradient:active-from-color (get-color "#c8c3de")
  "`From' color of active frames in `gradient' frame style."
  :type color
  :group (appearance gradient) 
  :after-set after-setting-frame-option
)

(defcustom gradient:active-to-color (get-color "#736c8c")
  "`To' color of active frames in `gradient' frame style."
  :type color
  :group (appearance gradient)
  :after-set after-setting-frame-option
)

(defcustom gradient:save-memory t
  "Use less memory when creating gradients, possibly affecting quality."
  :type boolean
  :group (appearance gradient)
  :after-set after-setting-frame-option
)

;; this is stuff that i've added to try to give more
;; configuration options. -rowan

(defcustom title-alignment 'center
  "Alignment of window title."
  :type symbol
  :options (left center right)
  :group (appearance gradient)
  :after-set after-setting-frame-option
)

(defcustom normal-font-color (get-color "#333333")
  "Color of unfocused text."
  :type color
  :group (appearance gradient)
  :after-set after-setting-frame-option
)

(defcustom active-font-color (get-color "#000000")
  "Color of focused text."
  :type color
  :group (appearance gradient)
  :after-set after-setting-frame-option
)

(let*
  (
    (render-bg 
      (lambda (img state)
        (apply 
          (cond 
            ((eq gradient:gradient-type 'diagonal) draw-diagonal-gradient)
            ((eq gradient:gradient-type 'horizontal) draw-horizontal-gradient)
            ((eq gradient:gradient-type 'vertical) draw-vertical-gradient)
          ) 
          img 
          (cond 
            ((eq state 'focused) (list gradient:active-from-color gradient:active-to-color))
            ((eq state 'highlighted) (list gradient:active-from-color gradient:active-to-color))
            ((eq state 'clicked) (list gradient:active-from-color gradient:active-to-color))
            ((eq state 'inactive-highlighted) (list gradient:normal-from-color gradient:normal-to-color))
            ((eq state 'inactive-clicked) (list gradient:normal-from-color gradient:normal-to-color))
            ((eq state nil) (list gradient:normal-from-color gradient:normal-to-color))
          )
        )

        (when (> (cdr (image-dimensions img)) 4) 
          (bevel-image img 1 t)
        )

        (set-image-border img 1 1 1 1)
      )
    )

    (scale 
      (lambda () 
        (if gradient:save-memory 2 1)
      )
    )

    ;; functions to control text
    (align-text 
      (lambda () 
        (cond 
          ((eq title-alignment 'left) '4)
          ((eq title-alignment 'center) 'center)
          ((eq title-alignment 'right) '-4)
        )
      )
    )

    (color-text 
      (lambda () 
        (list normal-font-color active-font-color)
      )
    )

    ;; buttons
    (button-shade 
      (list 
        (make-image "buttons/top_off.png") 
        (make-image "buttons/top.png")
        nil 
        (make-image "buttons/top_in.png")
      )
    )

    (button-minimize 
      (list 
        (make-image "buttons/bottom_off.png") 
        (make-image "buttons/bottom.png")
        nil 
        (make-image "buttons/bottom_in.png")
      )
    )

    (button-maximize
      (list
        (make-image "buttons/block_off.png")
        (make-image "buttons/block.png")
        nil
        (make-image "buttons/block_in.png")
      )
    )

    (button-close 
      (list 
        (make-image "buttons/box_off.png") 
        (make-image "buttons/box.png")
        nil 
        (make-image "buttons/box_in.png")
      )
    )

    ;; borders
    (border-line 
      (list
        (get-color "#444444")
        (get-color "#000000")
        nil
        nil
      )
    )
    (border-line2
      (list
        (get-color "#444444")
        (get-color "#444444")
        nil
        nil
      )
    )

    ;; title
    (title-right (make-image "title_right.png"))
    (title-trough (make-image "title_trough.png"))

    ;; frame definitions
    (frame 
      `(
        ;; titlebar pieces
        (
          (renderer . ,render-bg)
          (render-scale . ,scale)
          (foreground . ,color-text)
          (text . ,window-name)
          (x-justify . ,align-text)
          (y-justify . center)
          (left-edge . 0)
          (right-edge . 54)
          (top-edge . -17)
          (height . 17)
          (class . title)
        )
        ;; if you're adding buttons ...
        ;; this is where they sit.
        ;; so make sure to resize it appropriately
        ;; and also, adjust the width of stuff
        ;; above so it doesn't overlap
        (
          (background . ,title-trough)
          (right-edge . 4)
          (top-edge . -17)
          (width . 50)
          (class . title)
        )
        (
          (background . ,title-right)
          (right-edge . 0)
          (top-edge . -17)
          (class . title)
        )

        ;; window buttons
        (
          (background . ,button-close)
          (right-edge . 3)
          (top-edge . -15)
          (class . close-button)
          ;(removable . t)
        )
        (
          (background . ,button-minimize)
          (right-edge . 37)
          (top-edge . -15)
          (class . iconify-button)
          (removable . t)
        )
        (
          (background . ,button-shade)
          (right-edge . 22)
          (top-edge . -15)
          (class . shade-button)
          (removable . t)
        )
        ;(
        ;  (background . ,button-maximize)
        ;  (right-edge . 22)
        ;  (top-edge . -15)
        ;  (class . maximize-button)
        ;  (removable . t)
        ;)

        ;; window borders
        (
          (background . ,border-line)
          (left-edge . -2) 
          (right-edge . -2)
          (top-edge . -19)
          (height . 1)
          (class . top-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (right-edge . -1)
          (top-edge . -18)
          (height . 1)
          (class . top-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (top-edge . -18)
          (width . 1)
          (height . 18)
          (class . left-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (top-edge . -17)
          (width . 1)
          (height . 17)
          (class . left-border)
        )
        (
          (background . ,border-line)
          (right-edge . -2)
          (top-edge . -18)
          (width . 1)
          (height . 18)
          (class . right-border)
        )
        (
          (background . ,border-line2)
          (right-edge . -1)
          (top-edge . -17)
          (width . 1)
          (height . 17)
          (class . right-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (top-edge . 0)
          (bottom-edge . -1)
          (width . 1)
          (class . left-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (top-edge . 0)
          (bottom-edge . 0)
          (width . 1)
          (class . left-border)
        )
        (
          (background . ,border-line)
          (right-edge . -2)
          (top-edge . 0)
          (bottom-edge . -1)
          (width . 1)
          (class . right-border)
        )
        (
          (background . ,border-line2)
          (right-edge . -1)
          (top-edge . 0)
          (bottom-edge . 0)
          (width . 1)
          (class . right-border)
        )
        (
          (background . ,border-line)
          (left-edge . 0)
          (right-edge . 0)
          (bottom-edge . -2)
          (height . 1)
          (class . bottom-border)
        )
        (
          (background . ,border-line2)
          (left-edge . 0)
          (right-edge . 0)
          (bottom-edge . -1)
          (height . 1)
          (class . bottom-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (bottom-edge . -2)
          (width . 2)
          (height . 2)
          (class . bottom-left-corner)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (bottom-edge . -1)
          (width . 1)
          (height . 1)
          (class . bottom-left-corner)
        )
        (
          (background . ,border-line)
          (right-edge . -2)
          (bottom-edge . -2)
          (width . 2)
          (height . 2)
          (class . bottom-right-corner)
        )
        (
          (background . ,border-line2)
          (right-edge . -1)
          (bottom-edge . -1)
          (width . 1)
          (height . 1)
          (class . bottom-right-corner)
        )
      )
    )
    ;; end frame

    (shaped-frame 
      `(
        ;; titlebar pieces
        (
          (renderer . ,render-bg)
          (render-scale . ,scale)
          (foreground . ,color-text)
          (text . ,window-name)
          (x-justify . ,align-text)
          (y-justify . center) 
          (left-edge . 0)
          (right-edge . 54)
          (top-edge . -17)
          (height . 17)
          (class . title)
        )
        ;; if you're adding buttons ...
        ;; this is where they sit.
        ;; so make sure to resize it appropriately
        ;; and also, adjust the width of stuff
        ;; above so it doesn't overlap
        (
          (background . ,title-trough)
          (right-edge . 4)
          (top-edge . -17)
          (width . 50)
          (class . title)
        )
        (
          (background . ,title-right)
          (right-edge . 0)
          (top-edge . -17)
          (class . title)
        )

        ;; window buttons
        (
          (background . ,button-close)
          (right-edge . 3)
          (top-edge . -15)
          (class . close-button)
          ;(removable . t)
        )
        (
          (background . ,button-minimize)
          (right-edge . 37)
          (top-edge . -15)
          (class . iconify-button)
          (removable . t)
        )
        ;(
        ;  (background . ,button-shade)
        ;  (right-edge . 22)
        ;  (top-edge . -15)
        ;  (class . shade-button)
        ;  (removable . t)
        ;)
        (
          (background . ,button-maximize)
          (right-edge . 22)
          (top-edge . -15)
          ;(class . maximize-button)
          (class . shade-button)
          (removable . t)
        )

        ;; window borders
        (
          (background . ,border-line)
          (left-edge . -2)
          (right-edge . -2)
          (top-edge . -19)
          (height . 1)
          (class . top-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (right-edge . -1)
          (top-edge . -18)
          (height . 1)
          (class . top-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (top-edge . -18)
          (width . 1)
          (height . 18)
          (class . left-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (top-edge . -17)
          (width . 1)
          (height . 17)
          (class . left-border)
        )
        (
          (background . ,border-line)
          (right-edge . -2)
          (top-edge . -18)
          (width . 1)
          (height . 18)
          (class . right-border)
        )
        (
          (background . ,border-line2)
          (right-edge . -1)
          (top-edge . -17)
          (width . 1)
          (height . 17)
          (class . right-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (right-edge . -2)
          (top-edge . 0)
          (height . 1)
          (class . title)
        )
      )
    )
    ;; end shaped-frame

    (transient-frame 
      `(
        ;; titlebar pieces
        (
          (renderer . ,render-bg)
          (render-scale . ,scale)
          (foreground . ,color-text)
          (text . ,window-name)
          (x-justify . ,align-text)
          (y-justify . center)
          (left-edge . 0)
          (right-edge . 20)
          (top-edge . -17)
          (height . 17)
          (class . title)
        )
        ;; transient windows should only have 
        ;; a close button so this trough will
        ;; be considerably smaller than normal
        (
          (background . ,title-trough)
          (right-edge . 4)
          (top-edge . -17)
          (width . 16)
          (class . title)
        )
        (
          (background . ,title-right)
          (right-edge . 0)
          (top-edge . -17)
          (class . title)
        )

        ;; window buttons
        (
          (background . ,button-close)
          (right-edge . 3)
          (top-edge . -15)
          (class . close-button)
          ;(removable . t)
        )

        ;; window borders
        (
          (background . ,border-line)
          (left-edge . -2)
          (right-edge . -2)
          (top-edge . -19)
          (height . 1)
          (class . top-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (right-edge . -1)
          (top-edge . -18)
          (height . 1)
          (class . top-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (top-edge . -18)
          (width . 1)
          (height . 18)
          (class . left-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (top-edge . -17)
          (width . 1)
          (height . 17)
          (class . left-border)
        )
        (
          (background . ,border-line)
          (right-edge . -2)
          (top-edge . -18)
          (width . 1)
          (height . 18)
          (class . right-border)
        )
        (
          (background . ,border-line2)
          (right-edge . -1)
          (top-edge . -17)
          (width . 1)
          (height . 17)
          (class . right-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (top-edge . 0)
          (bottom-edge . -1)
          (width . 1)
          (class . left-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (top-edge . 0)
          (bottom-edge . 0)
          (width . 1)
          (class . left-border)
        )
        (
          (background . ,border-line)
          (right-edge . -2)
          (top-edge . 0)
          (bottom-edge . -1)
          (width . 1)
          (class . right-border)
        )
        (
          (background . ,border-line2)
          (right-edge . -1)
          (top-edge . 0)
          (bottom-edge . 0)
          (width . 1)
          (class . right-border)
        )
        (
          (background . ,border-line)
          (left-edge . 0)
          (right-edge . 0)
          (bottom-edge . -2)
          (height . 1)
          (class . bottom-border)
        )
        (
          (background . ,border-line2)
          (left-edge . 0)
          (right-edge . 0)
          (bottom-edge . -1)
          (height . 1)
          (class . bottom-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (bottom-edge . -2)
          (width . 2)
          (height . 2)
          (class . bottom-left-corner)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (bottom-edge . -1)
          (width . 1)
          (height . 1)
          (class . bottom-left-corner)
        )
        (
          (background . ,border-line)
          (right-edge . -2)
          (bottom-edge . -2)
          (width . 2)
          (height . 2)
          (class . bottom-right-corner)
        )
        (
          (background . ,border-line2)
          (right-edge . -1)
          (bottom-edge . -1)
          (width . 1)
          (height . 1)
          (class . bottom-right-corner)
        )
      )
    )
    ;; end transient-frame

    (shaped-transient-frame 
      `(
        ;; titlebar pieces
        (
          (renderer . ,render-bg)
          (render-scale . ,scale)
          (foreground . ,color-text)
          (text . ,window-name)
          (x-justify . ,align-text)
          (y-justify . center)
          (left-edge . 0)
          (right-edge . 20)
          (top-edge . -17)
          (height . 17)
          (class . title)
        )
        ;; transient windows should only have 
        ;; a close button so this trough will
        ;; be considerably smaller than normal
        (
          (background . ,title-trough)
          (right-edge . 4)
          (top-edge . -17)
          (width . 16)
          (class . title)
        )
        (
          (background . ,title-right)
          (right-edge . 0)
          (top-edge . -17)
          (class . title)
        )

        ;; window buttons
        (
          (background . ,button-close)
          (right-edge . 3)
          (top-edge . -15)
          (class . close-button)
          ;(removable . t)
        )

        ;; window borders
        (
          (background . ,border-line)
          (left-edge . -2)
          (right-edge . -2)
          (top-edge . -19)
          (height . 1)
          (class . top-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (right-edge . -1)
          (top-edge . -18)
          (height . 1)
          (class . top-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (top-edge . -18)
          (width . 1)
          (height . 18)
          (class . left-border)
        )
        (
          (background . ,border-line2)
          (left-edge . -1)
          (top-edge . -17)
          (width . 1)
          (height . 17)
          (class . left-border)
        )
        (
          (background . ,border-line)
          (right-edge . -2)
          (top-edge . -18)
          (width . 1)
          (height . 18)
          (class . right-border)
        )
        (
          (background . ,border-line2)
          (right-edge . -1)
          (top-edge . -17)
          (width . 1)
          (height . 17)
          (class . right-border)
        )
        (
          (background . ,border-line)
          (left-edge . -2)
          (right-edge . -2)
          (top-edge . 0)
          (height . 1)
          (class . title)
        )
      )
    )
    ;; end shaped-transient-frame
  )

  (def-frame-class shade-button '((cursor . left_ptr))
    (bind-keys shade-button-keymap "Button1-Off" 'toggle-window-shaded)
  )

  (add-frame-style 'GleamThin
    (lambda (w type)
      (cond 
        ((eq type 'shaped) shaped-frame)
        ((eq type 'shaded) shaped-frame)
        ((eq type 'transient) transient-frame)
        ((eq type 'shaped-transient) shaped-transient-frame)
        ((eq type 'shaded-transient) shaped-transient-frame)
        ((eq type 'unframed) nil-frame)
        (t frame)
      )
    )
  )
)
;; end
